package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.client.UserServiceClient;
import com.diagbot.dto.GetLibraryDetailListDTO;
import com.diagbot.dto.LibraryDetailDTO;
import com.diagbot.dto.LibraryIndexDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.LibraryDetail;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.LibraryDetailService;
import com.diagbot.service.impl.LibraryDetailServiceImpl;
import com.diagbot.util.DateUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.AddLibraryDetailVO;
import com.diagbot.vo.GetLibraryDetailListVO;
import com.diagbot.vo.GetLibraryDetailVO;
import com.diagbot.vo.LibraryIndexVO;
import com.diagbot.vo.RemoveLibraryDetailVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: 术语基本信息查询业务层
 * @author: Weixuan Huang
 * @time: 2019/1/16 14:17
 */
@Component
public class LibraryDetailFacade extends LibraryDetailServiceImpl {

    @Autowired
    private UserServiceClient userServiceClient;
    @Autowired
    @Qualifier("libraryDetailServiceImpl")
    private LibraryDetailService libraryDetailService;

    /**
     * 获取医学术语命名列表
     *
     * @param getLibraryDetailListVO
     * @return
     */
    public IPage<GetLibraryDetailListDTO> getLibraryDetailList(GetLibraryDetailListVO getLibraryDetailListVO) {
        IPage<GetLibraryDetailListDTO> ipage = this.baseMapper.getLibraryDetailList(getLibraryDetailListVO);
        List<String> ids = ipage.getRecords().stream().map(i -> i.getModifier()).distinct().collect(Collectors.toList());
        if (ids.size() > 0) {
            RespDTO<Map<String, String>> respDTO = userServiceClient.getUserInfoByIds(ids);
            if (respDTO == null || !CommonErrorCode.OK.getCode().equals(respDTO.code)) {
                throw new CommonException(CommonErrorCode.RPC_ERROR,
                        "获取用户信息失败");
            }
            ipage.getRecords().forEach(i -> {
                i.setModifier(respDTO.data.get(i.getModifier()));
            });
        }
        return ipage;
    }

    /**
     * 医学术语静态知识删除
     *
     * @param removeLibraryDetailVO
     * @return
     */
    public Boolean removeLibraryDetail(RemoveLibraryDetailVO removeLibraryDetailVO) {
        if (!(IsDeleteEnum.N.getKey().equals(removeLibraryDetailVO.getStatus())
                || IsDeleteEnum.Y.getKey().equals(removeLibraryDetailVO.getStatus()))) {
            throw new CommonException(CommonErrorCode.STATUS_IS_ERROR, "状态值错误【Y】或【N】");
        }
        String person = UserUtils.getCurrentPrincipleID();
        List<Long> ids = removeLibraryDetailVO.getConceptId();
        Date now = DateUtil.now();
        this.update(new LibraryDetail(), new UpdateWrapper<LibraryDetail>()
                .in("concept_id", ids)
                .set("gmt_modified", now)
                .set("modifier", person)
                .set("is_deleted", removeLibraryDetailVO.getStatus()));
        return true;
    }

    /**
     * 获取医学术语静态知识详情
     *
     * @param getLibraryDetailVO
     * @return
     */
    public List<LibraryDetailDTO> getLibraryDetail(GetLibraryDetailVO getLibraryDetailVO) {
        return this.getByParam(getLibraryDetailVO.getConceptId());
    }

    /**
     * 医学术语静态知识添加或者编辑
     *
     * @param libraryDetailVOList
     * @return
     */
    public Boolean addLibraryDetail(List<AddLibraryDetailVO> libraryDetailVOList) {
        if (ListUtil.isEmpty(libraryDetailVOList)) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
                    "静态知识明细内容不能为空");
        }

        // 删除原明细信息
        Long conceptId = libraryDetailVOList.get(0).getConceptId();
        if (conceptId != null) {
            this.remove(new QueryWrapper<LibraryDetail>()
                    .eq("concept_id", conceptId));
        }

        //  新增明细信息
		String currentUser = UserUtils.getCurrentPrincipleID();
        Date now = DateUtil.now();
        List<LibraryDetail> list = new ArrayList<>();
        for (AddLibraryDetailVO detailVO : libraryDetailVOList) {
            LibraryDetail bean = new LibraryDetail();
            bean.setConceptId(detailVO.getConceptId());
            bean.setGmtModified(now);
            bean.setGmtCreate(now);
            bean.setIsDeleted(IsDeleteEnum.N.getKey());
            bean.setModifier(currentUser);
            bean.setCreator(currentUser);
            bean.setContent(detailVO.getContent());
            bean.setIsReason(detailVO.getIsReason());
            bean.setOrderNo(detailVO.getOrderNo());
            bean.setPosition(detailVO.getPosition());
            bean.setText(detailVO.getText());
            bean.setTitle(detailVO.getTitle());
            list.add(bean);
        }
        libraryDetailService.saveBatch(list);
        return true;
    }


    /**
     * 检索
     *
     * @param libraryIndexVO
     * @return
     */
    public List<LibraryIndexDTO> indexFac(LibraryIndexVO libraryIndexVO) {
        return this.index(libraryIndexVO);
    }

}
